%%! -*- coding:utf-8; mode:latex; -*-


%% Copyright (c) 2014 Moritz Klammler
%%
%% Permission is  hereby granted, free  of charge,  to any person  obtaining a
%% copy of this software and  associated documentation files (the "Software"),
%% to deal in  the Software without restriction,  including without limitation
%% the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
%% and/or  sell copies  of the  Software, and  to permit  persons to  whom the
%% Software is furnished to do so, subject to the following conditions:
%%
%% The above copyright notice and this  permission notice shall be included in
%% all copies or substantial portions of the Software.
%%
%% THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
%% IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
%% FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
%% THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
%% LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
%% FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
%% DEALINGS IN THE SOFTWARE.


\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{bsc-klammler}[2014/10/24 XeLaTeX class for my bachelor thesis]

\LoadClass[11pt,a4paper,twoside]{report}

\providecommand*{\BIBLATEXBACKEND}{biber}
\providecommand*{\PRINTVERSION}{false}

\RequirePackage{etoolbox}
\RequirePackage{xltxtra}
\RequirePackage{polyglossia}

\newtoggle{bsc@klammler@print}
\settoggle{bsc@klammler@print}{\PRINTVERSION}

\RequirePackage[ttscale=0.875]{libertine}
\RequirePackage[libertine]{newtxmath}

\setmainfont[Numbers={OldStyle}]{Linux Libertine O}

\setmainlanguage{english}

\providecommand*{\sbseries}{%
  \fontspec[Numbers={OldStyle}]{Linux Libertine Semibold}%
}
\providecommand*{\textsb}[1]{\bgroup\sbseries#1\egroup}
\let\strong\textsb

\newenvironment{atletter}{%
  \makeatletter%
}{%
  \makeatother%
}

% The following macros superseed the builtin '\romannumeral', '\roman' et al
% macros by selecting the special glyph from the Libertine font instead of
% faking the numeral using ordinary letters.  Especially for 'III' (which looks
% ugly due to the 'II' ligature) this makes quite a difference.  The magic part
% of this is courtesy of Leo Liu:
%
%     https://tex.stackexchange.com/questions/38695/using-unicode-roman-numerals-in-xetex
%
\newcommand*{\uromannumeral}[1]{\symbol{\numexpr"216F+#1\relax}}
\newcommand*{\uRomannumeral}[1]{\symbol{\numexpr"215F+#1\relax}}
\newcommand*{\uroman}[1]{\uromannumeral{\the\value{#1}}}
\newcommand*{\uRoman}[1]{\uRomannumeral{\the\value{#1}}}

\RequirePackage{booktabs}
\RequirePackage{tabularx}
\RequirePackage[newcommands]{ragged2e}

\RequirePackage{tikz}
\usetikzlibrary{arrows}
\usetikzlibrary{calc}
\usetikzlibrary{patterns}
\usetikzlibrary{positioning}

\RequirePackage{tango-colors}
\RequirePackage{gnuplot-lua-tikz}

\newcommand*{\inputplot}[2][\linewidth]{\input{plots/#2.tikz}}

\colorlet{gp lt color 0}{TangoSkyBlue2}
\colorlet{gp lt color 1}{TangoScarletRed2}
\colorlet{gp lt color 2}{TangoChameleon2}
\colorlet{gp lt color 3}{TangoPlum2}
\colorlet{gp lt color 4}{TangoOrange2}
\colorlet{gp lt color 5}{TangoChocolate2}
\colorlet{gp lt color 6}{TangoButter2}
\colorlet{gp lt color 7}{TangoAluminium6}

\newcommand*{\fancykilo}[2][]{\ifstrequal{#1}{0}{$0$}{$#2\,\text{k}$}}
\newcommand*{\fancydecpower}[3][]{\ifstrequal{#1}{0}{$0$}{$#2\cdot10^{#3}$}}
\newcommand*{\ylabelfrac}[2]{#1\enspace/\enspace#2}

\newcommand*{\plotkeyfont}{}

\tikzset{%
  auxcoord/.style= {
    circle,
    inner sep = 0pt,
    minimum size = 3pt,
    fill = orange,
  },
  literal/.style = {
    rectangle,
    ultra thin,
    fill = TangoAluminium1,
    draw = TangoAluminium2,
    text = TangoAluminium6,
    font = \normalfont\small\ttfamily,
    text height = 1.5ex,
    text depth = 0.5ex,
    inner xsep = 1pt,
    inner ysep = 0pt,
  },
  nonzero/.style = {
    fill = TangoSkyBlue1,
    draw = TangoSkyBlue3,
    ultra thin,
  },
  nonzero1/.style = {
    fill = TangoScarletRed1,
    draw = TangoScarletRed3,
    ultra thin,
  },
  nonzero2/.style = {
    fill = TangoChameleon1,
    draw = TangoChameleon3,
    ultra thin,
  },
  nonzero3/.style = {
    fill = TangoPlum1,
    draw = TangoPlum3,
    ultra thin,
  },
  nonzero0/.style = {
    fill = TangoAluminium3,
    draw = TangoAluminium5,
    ultra thin,
  },
  outline/.style = {
    draw = TangoAluminium6,
    very thin,
  },
  fancyarrow/.style = {
      draw,
      thick,
      > = latex,
  },
}

\tikzset{%
  umlclass/.style = {
    rectangle,
    draw,
    font = \ttfamily,
  },
  umlassociation/.style = {
    draw,
  },
  umlaggregation/.style = {
    draw,
    > = open diamond,
  },
  umlmultiplicity/.style = {
    inner sep = 1pt,
    outer sep = 5pt,
    font = \small,
  },
  uml@inheritance/.style = {
    draw,
    > = open triangle 90,
  },
  umlextends/.style = {
    uml@inheritance,
  },
  umlimplements/.style = {
    uml@inheritance,
    dashed,
  },
}

\newcommand*{\umlAbstract}[1]{\textit{#1}}
\newcommand*{\umlStatic}[1]{\underline{#1}}
\newcommand*{\uml@access}[1]{\ensuremath{#1}}
\newcommand*{\umlPrivate}{\uml@access{-}}
\newcommand*{\umlPublic}{\uml@access{+}}
\newcommand*{\umlProtected}{\uml@access{\#}}
\newcommand*{\uml@tag}[1]{\mbox{\small«#1»}}
\newcommand*{\umlInterfaceTag}{\uml@tag{interface}}
\newcommand*{\umlmultiplicity}[2]{%
  \ifstrequal{#1}{#2}{\ensuremath{#1}}{\ensuremath{#1..#2}}%
}

\newcommand*{\literal}[1]{\tikz[baseline=(l.base)]{\node[literal](l){#1};}}

\newcommand*{\quotesymbol}[1]{\mbox{“\hspace{1pt}#1\hspace{1pt}”}}

\DeclareRobustCommand*{\colorprobe}[1]{%
  \quotesymbol{\textcolor{#1}{\rule{1.5ex}{1.5ex}}\hspace{1pt}}%
}

\RequirePackage[backend=\BIBLATEXBACKEND]{biblatex}
\addbibresource{literature.bib}

\RequirePackage{float}
\RequirePackage{caption}

\newfloat{codelisting}{htbp}{lst}[chapter]
\floatname{codelisting}{Listing}

\RequirePackage{mythm}
\mynewthm{definition}{Definition}

\renewcommand*{\captionfont}{\normalfont}
\renewcommand*{\captionlabelfont}{\normalfont\scshape}
\let\mythmfont\captionfont
\let\mythmtitlefont\captionlabelfont

\RequirePackage{listings}
\RequirePackage{pseudo}
\lstset{%
  mathescape=true,%
  basicstyle={\normalfont\ttfamily\small},%
  keywordstyle=[1]{\sbseries},%
  commentstyle={\itshape},%
  stringstyle={},%
}

\lstdefinestyle{c++11}{%
  language = {C++},
  morekeywords={%
    auto,%
    constexpr,%
    noexcept,%
    restrict,%
  },
  moredelim=**[l][\color{TangoAluminium5}]{\#},%
}

\RequirePackage{pifont}
\newcommand*{\Yes}{\textcolor{TangoChameleon3}{\ding{52}}}
\newcommand*{\No}{\textcolor{TangoScarletRed3}{\ding{56}}}
\newcommand*{\SemiYes}{\textcolor{TangoOrange3}{\ding{51}}}
\newcommand*{\SemiNo}{\textcolor{TangoOrange3}{\ding{55}}}
\newcommand*{\Dunno}{?}

\newcommand*{\supertrademark}{\raisebox{1ex}{\scriptsize\textregistered}}

\RequirePackage{hyperref}

\iftoggle{bsc@klammler@print}{%
  \hypersetup{hidelinks}
}{%
  \hypersetup{%
    colorlinks,
    linktoc = page,
    linkcolor = TangoScarletRed3,
    citecolor = TangoChameleon3,
    filecolor = TangoSkyBlue3,
    urlcolor = TangoPlum3,
  }
}

\newcommand*{\code}[1]{\mbox{\texttt{#1}}}
% Comment out the previous and uncomment the following line to mark all
% codes with color such as to make them more visible while debugging.
%\newcommand*{\code}[1]{\mbox{\textcolor{TangoChocolate3}{\texttt{#1}}}}
\newcommand*{\spelled}[1]{\underline{#1}}
\newcommand*{\algo}[1]{\textsc{#1}}

\providecommand*{\abs}[1]{\lvert#1\rvert}
\providecommand*{\norm}[1]{\lVert#1\rVert}
\let\card\abs
\newcommand*{\floor}[1]{\lfloor#1\rfloor}
\newcommand*{\ceil}[1]{\lceil#1\rceil}
\newcommand*{\trans}{^\text{T}}
\newcommand*{\inv}{^{-1}}
\newcommand*{\invtrans}{^{-\text{T}}}
\providecommand*{\lequiv}{\Leftrightarrow}
\providecommand*{\lantiv}{\nLeftrightarrow}
\providecommand*{\limplies}{\Rightarrow}

% TODO: These are ugly.
\newcommand*{\lhsdef}{:=}
\newcommand*{\rhsdef}{=:}

% This denotes a non-scalar quantity.
\newcommand*{\ns}[1]{\boldsymbol{#1}}

\newcommand*{\LU}{\ns{L}\ns{U}}
\newcommand*{\LLT}{\ns{L}\ns{L}\trans}
\newcommand*{\LDU}{\ns{L}\ns{D}\ns{U}}
\newcommand*{\LDLT}{\ns{L}\ns{D}\ns{L}\trans}
\providecommand*{\Omikron}{O}
\newcommand*{\LandauBigOmikron}{\Omikron}
\newcommand*{\LandauBigOmega}{\Omega}
\newcommand*{\LandauBigTheta}{\Theta}

\DeclareMathOperator{\row}{row}
\DeclareMathOperator{\col}{col}
\DeclareMathOperator{\rows}{rows}
\DeclareMathOperator{\cols}{cols}
\DeclareMathOperator{\size}{size}
\DeclareMathOperator{\NZC}{NZC}

\newcommand{\bra}[2][\relax]{%
  \ifx\relax#1\relax%
    \left\langle#2\right|%
  \else%
    \left\langle#2\middle|#1\right|%
  \fi%
}

\newcommand{\ket}[2][\relax]{%
  \ifx\relax#1\relax%
    \left|#2\right\rangle%
  \else%
    \left|#1\middle|#2\right\rangle%
  \fi%
}

\newcommand{\braket}[3][\relax]{%
  \ifx\relax#1\relax%
    \left\langle#2\middle|#3\right\rangle%
  \else%
    \left\langle#2\middle|#1\middle|#3\right\rangle%
  \fi%
}

\providecommand*{\born}{\ensuremath{*}}
\providecommand*{\died}{\ensuremath{\dag}}

\newcommand{\preface}{%
  \cleardoublepage%
  \pagestyle{headings}%
  \setcounter{page}{1}%
  \renewcommand*{\thepage}{\roman{page}}%
}

\newcommand{\mainpart}{%
  \cleardoublepage%
  \setcounter{page}{1}%
  \renewcommand*{\thepage}{\arabic{page}}%
}

\newcommand{\trapniam}{%
  \cleardoublepage%
  \appendix
  \setcounter{page}{1}%
  \renewcommand*{\thepage}{\Roman{page}}%
}

\setcounter{secnumdepth}{4}
\setcounter{tocdepth}{4}
\renewcommand*{\thepage}{\roman{page}}%
\pagestyle{empty}

\renewenvironment{titlepage}{%
  \thispagestyle{empty}%
  \setlength{\parindent}{0em}%
}{%
  \clearpage
}

\renewcommand{\maketitle}{%
  \begin{titlepage}
    \large\noindent\leavevmode%
    \begin{tabularx}{\textwidth}{lXr}
      \pgfimage[height=21mm]{pics/logo-kit.pdf}&
      &
      \pgfimage[height=14mm]{pics/logo-iti.pdf}
    \end{tabularx}
    \par\vspace{\stretch{2}}
    \begin{tabularx}{\textwidth}{X}
      \Large\@author\\[2ex]
      \toprule
      \vspace{1ex}
      \LARGE\bfseries\raggedright\@title\\[2ex]
      \bottomrule
    \end{tabularx}
    \par\vspace{\stretch{1}}
    \begin{center}
      Bachelor's Thesis\par\bigskip
      Date of Submission: \@date
    \end{center}
    \par\vspace{\stretch{3}}
    \begin{tabular}{ll}
      Supervisor:           & Prof.~Dr.~Peter Sanders\\
                            & Dipl.-Inform., Dipl.-Math.~Jochen Speck\\[2ex]
      Secondary Supervisor: & Prof.~Dr.~Christian Wieners\\
                            & Dr.~Daniel Weiß
    \end{tabular}
    \par\vspace{\stretch{2}}
    \begin{tabular}{l}
      Institute of Theoretical Informatics, Algorithmics\\
      Department of Informatics\\[2ex]
      Institute for Applied and Numerical Mathematics\\
      Department of Mathematics\\[3ex]
      Karlsruhe Institute of Technology
    \end{tabular}
  \end{titlepage}
}
